// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "JSONLoader.hpp"

using namespace Dawn;

const std::string JSONLoader::ASSET_TYPE = "json";

JSONLoader::JSONLoader(
  const std::shared_ptr<AssetManager> assetManager,
  const std::string name
) :
  AssetLoader(assetManager, name),
  loader(name),
  state(JSONLoaderState::INITIAL)
{
  this->typetest = this->getAssetType();
}

void JSONLoader::updateAsync() {
  if(this->state != JSONLoaderState::INITIAL) return;
  
  this->state = JSONLoaderState::LOADING_JSON;
  std::string jsonContents = loader.getEntireContentsAsString();
  this->data = json::parse(jsonContents);
  this->state = JSONLoaderState::DONE;
  this->loaded = true;
}

void JSONLoader::updateSync() {
}

std::string JSONLoader::getAssetType() const {
  return JSONLoader::ASSET_TYPE;
}

JSONLoader::~JSONLoader() {
  
}

