// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "asset/AssetLoader.hpp"
#include "asset/AssetDataLoader.hpp"
#include "display/shader/ShaderManager.hpp"
#include "display/shader/ShaderProgram.hpp"

namespace Dawn {
  enum class ShaderLoaderState {
    INITIAL,
    LOADING,
    LOADED
  };

  class ShaderLoader : public AssetLoader {
    protected:
      enum ShaderLoaderState state;
      std::shared_ptr<ShaderProgram> shader;

    public:
      const static std::string ASSET_TYPE;

      Slang::ComPtr<IComponentType> linkedProgram;
      Slang::ComPtr<IComponentType> program;

      IModule* module;

      ShaderLoader(
        const std::shared_ptr<AssetManager> assetManager,
        const std::string name
      );
      void updateSync() override;
      void updateAsync() override;
      std::string getAssetType() const override;
      
      /**
       * Retreives the shader program for this loader.
       * 
       * @return The shader program that this loader is managing.
       */
      std::shared_ptr<ShaderProgram> getShader();

      ~ShaderLoader();
  };
}