// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "asset/AssetLoader.hpp"
#include "asset/AssetDataLoader.hpp"
#include "display/Texture.hpp"

#define TEXTURE_LOADER_HEADER_SIZE 256

namespace Dawn {
  enum class TextureLoaderLoadState {
    INITIAL,
    ASYNC_LOADING,
    ASYNC_DONE,
    SYNC_LOADING,
    SYNC_DONE
  };

  class TextureLoader : public AssetLoader {
    protected:
      AssetDataLoader loader;
      enum TextureLoaderLoadState state;
      uint8_t *data = nullptr;
      int32_t width = -1, height = -1;
      enum TextureFormat format;
      enum TextureWrapMode wrapX;
      enum TextureWrapMode wrapY;
      enum TextureFilterMode filterMin;
      enum TextureFilterMode filterMag;
      std::shared_ptr<Texture> texture;

    public:
      const static std::string ASSET_TYPE;

      TextureLoader(
        const std::shared_ptr<AssetManager> assetManager,
        const std::string name
      );

      void updateSync() override;
      void updateAsync() override;
      std::string getAssetType() const override;

      /**
       * Get the texture asset.
       * 
       * @return Texture asset.
       */
      std::shared_ptr<Texture> getTexture();
      ~TextureLoader();
  };
}
