// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PrefabLoader.hpp"
#include "SceneLoader.hpp"
#include "game/Game.hpp"
#include "assert/assert.hpp"
#include "asset/loader/TextureLoader.hpp"
#include "scene/Scene.hpp"
#include "component/SceneComponentRegistry.hpp"
#include "util/JSON.hpp"

using namespace Dawn;

const std::string PrefabLoader::ASSET_TYPE = "prefab";

PrefabLoader::PrefabLoader(
  const std::shared_ptr<AssetManager> assetManager,
  const std::string name
) :
  LoaderForSceneItems(assetManager, name)
{
  this->typetest = this->getAssetType();
}

void PrefabLoader::updateSync() {
  switch(this->state) {
    case PrefabLoaderState::INITIAL:
      jsonLoader = getAssetManager()->get<JSONLoader>(this->name);
      this->state = PrefabLoaderState::LOADING_JSON;
      break;
    
    case PrefabLoaderState::LOADING_JSON:
      assertNotNull(this->jsonLoader, "JSON Loader is NULL?");
      if(!this->jsonLoader->loaded) return;
      this->state = PrefabLoaderState::LOADING_DEPENDENCIES;
      this->ctx->data = this->jsonLoader->data;
      this->setupDependencies();
      break;

    case PrefabLoaderState::LOADING_DEPENDENCIES:
      assertTrue(this->jsonLoader->loaded, "JSON loader not loaded?");

      // Check if all dependencies are loaded.
      for(auto &asset : ctx->assets) {
        if(!asset.second->loaded) return;
      }

      this->state = PrefabLoaderState::DEPENDENCIES_LOADED;
      this->loaded = true;
      break;

    default:
      break;
  }
}

void PrefabLoader::updateAsync() {
}

std::string PrefabLoader::getAssetType() const {
  return PrefabLoader::ASSET_TYPE;
}

PrefabLoader::~PrefabLoader() {

}