// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/pass/RenderPass.hpp"
#include "display/pass/RenderPassContext.hpp"
#include "display/mesh/Mesh.hpp"

namespace Dawn {
  class IRenderableComponent {
    public:
      /**
       * Retreive the list of render passes for this component.
       * 
       * @param ctx Context for the render pass.
       * @return List of render passes.
       */
      virtual std::vector<std::shared_ptr<RenderPass>> getPasses(
        struct RenderPassContext &ctx
      ) = 0;
  };

  /**
   * Short-hand function to create a render pass.
   * 
   * @tparam S Shader type.
   * @tparam D Shader's data type
   * @param self Instance of the IRenderableComponent that is creating the pass.
   * @param data Data to use for the render pass.
   * @return Created render pass.
   */
  // template<class S, typename D>
  // std::shared_ptr<IRenderPass> createRenderPass(
  //   SceneComponent &self,
  //   const D data,
  //   const std::unordered_map<
  //     shadertexturebinding_t, std::shared_ptr<Texture>
  //   > textures = {},
  //   const std::shared_ptr<Mesh> mesh = nullptr,
  //   const enum MeshDrawMode drawMode = MeshDrawMode::TRIANGLES,
  //   int32_t indiceStart = 0,
  //   int32_t indiceCount = -1
  // ) {
  //   return std::make_shared<RenderPass<S,D>>(
  //     self,
  //     data,
  //     textures,
  //     mesh,
  //     drawMode,
  //     indiceStart,
  //     indiceCount
  //   );
  // }
}