// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SimpleTexturedMaterial.hpp"
#include "util/JSON.hpp"
#include "asset/loader/TextureLoader.hpp"
#include "asset/loader/ShaderLoader.hpp"

using namespace Dawn;

void SimpleTexturedMaterial::initShaderPrograms() {
  this->shader = getGame()->assetManager->get<ShaderLoader>("shaders/hello-world.slang")->getShader();
}

void SimpleTexturedMaterial::load(std::shared_ptr<SceneLoadContext> ctx) {
  if(ctx->data.contains("color")) {
    this->setColor(JSON::color(ctx->data["color"]));
  }

  if(ctx->data.contains("texture")) {
    auto asset = ctx->getAsset<TextureLoader>(
      ctx->data["texture"].get<std::string>()
    );
    this->setTexture(asset->getTexture());
  }
}

struct Color SimpleTexturedMaterial::getColor() {
  return this->data.color;
}

std::shared_ptr<Texture> SimpleTexturedMaterial::getTexture() {
  return this->data.texture;
}

void SimpleTexturedMaterial::setTexture(
  const std::shared_ptr<Texture> texture
) {
  this->data.texture = texture;
}

void SimpleTexturedMaterial::setColor(const struct Color color) {
  this->data.color = color;
}

std::vector<std::shared_ptr<RenderPass>> SimpleTexturedMaterial::getPasses(
  struct RenderPassContext &ctx
) {
  // this->data.model = this->getItem()->getWorldTransform();
  // this->data.projection = ctx.camera->getProjection();
  // this->data.view = ctx.camera->getItem()->getWorldTransform();
  // auto textures = std::unordered_map<
  //   shadertexturebinding_t, std::shared_ptr<Texture>
  // >();
  
  // if(this->texture) {
  //   this->data.hasTexture = true;
  //   this->data.texture = 0;
  //   textures[this->data.texture] = this->texture;
  // } else {
  //   this->data.data.hasTexture = false;
  // }

  return {
    std::make_shared<RenderPass>(
      *this,
      nullptr,// Get mesh automatically.
      MeshDrawMode::TRIANGLES,// Move this later.
      0,// Move this later.
      -1,// Move this later.
      this->shader
    )
  };
}
