// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "QuadMeshComponent.hpp"
#include "display/mesh/QuadMesh.hpp"
#include "util/JSON.hpp"

using namespace Dawn;

void QuadMeshComponent::meshBuffer(std::shared_ptr<Mesh> mesh) {
  mesh->createBuffers(QUAD_VERTICE_COUNT, QUAD_INDICE_COUNT);
  QuadMesh::buffer(
    mesh,
    this->positions,
    this->uvs,
    0, 0
  );
}

void QuadMeshComponent::meshLoad(std::shared_ptr<SceneLoadContext> ctx) {
  if(ctx->data.contains("positions")) {
    this->positions = JSON::vec4(ctx->data["positions"]);
  } else {
    this->positions = glm::vec4(0.0f, 0.0f, 1.0f, 1.0f);
  }

  if(ctx->data.contains("uvs")) {
    this->uvs = JSON::vec4(ctx->data["uvs"]);
  } else {
    this->uvs = glm::vec4(0.0f, 0.0f, 1.0f, 1.0f);
  }
}

void QuadMeshComponent::setPositions(const glm::vec4 &positions) {
  this->positions = positions;
  this->invalidate();
}

void QuadMeshComponent::setUVs(const glm::vec4 &uvs) {
  this->uvs = uvs;
  this->invalidate();
}