// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/shader/ShaderStage.hpp"

namespace Dawn {

  class IShaderProgram {
    protected:
      std::vector<std::shared_ptr<ShaderStage>> stages;

    public:
      /**
       * Initialize the ShaderProgram. In your render hosts' implementation
       * this would initialize the shader on your GPU and ideally set the
       * initial values for uniforms and objects.
       * 
       * Provided stages will already be "initialized" (at least as far as the
       * engine is concerned), and should be ready to be used. If your render
       * host needs the program to be initialized prior to the stages you will
       * need to modify how stages are initialized, this is by design as we are
       * loading the shader code and "initializing" the stages before we create
       * and initialize the program which will use and link the stages.
       * 
       * @param stages The stages to use in this program.
       */
      virtual void init(
        const std::vector<std::shared_ptr<ShaderStage>> &stages
      );

      /**
       * Destroy the IShaderProgram2 object
       */
      virtual ~IShaderProgram();
  };
}