// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "ShaderManager.hpp"
#include "assert/assert.hpp"

using namespace Dawn;


ShaderManager::ShaderManager() :
  targetDescription(),
  sessionDescription()
{
}

void ShaderManager::init(const std::shared_ptr<Game> &game) {
  assertNotNull(game, "Game instance must not be null.");
  
  this->game = game;

  // Create the file system
  this->fileSystem.sm = shared_from_this();

  // Create the global session
  createGlobalSession(globalSession.writeRef());

  // Set the target description, TODO: interface
  targetDescription.format = SLANG_GLSL;
  targetDescription.profile = globalSession->findProfile("glsl_330");

  // Set the session description
  sessionDescription.targets = &targetDescription;
  sessionDescription.targetCount = 1;
  sessionDescription.searchPathCount = 0;
  sessionDescription.fileSystem = &this->fileSystem;

  // Create session
  globalSession->createSession(sessionDescription, session.writeRef());
}

std::shared_ptr<Game> ShaderManager::getGame() {
  auto game = this->game.lock();
  assertNotNull(game, "Game instance must not be null.");
  return game;
}

ShaderManager::~ShaderManager() {
}