// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "ShaderManagerSlangFileSystem.hpp"
using namespace slang;

namespace Dawn {
  class Game;
  class ShaderLoader;

  class ShaderManager : public std::enable_shared_from_this<ShaderManager> {
    private:
      std::weak_ptr<Game> game;
      ShaderManagerSlangFileSystem fileSystem;
      Slang::ComPtr<IGlobalSession> globalSession;
      TargetDesc targetDescription;
      SessionDesc sessionDescription;
      Slang::ComPtr<ISession> session;

    public:
      /**
       * Creates a new shader manager.
       */
      ShaderManager();

      /**
       * Initializes the shader manager.
       * 
       * @param game The game instance that the shader manager is being used in.
       */
      void init(const std::shared_ptr<Game> &game);

      /**
       * Retreives the game instance.
       * 
       * @return Game instance.
       */
      std::shared_ptr<Game> getGame();

      /**
       * Disposes of all shaders.
       */
      ~ShaderManager();

      friend class ShaderLoader;
  };
}