// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "ShaderManagerSlangFileSystem.hpp"
#include "assert/assert.hpp"
#include "display/shader/ShaderManager.hpp"
#include "game/Game.hpp"
#include "asset/loader/StringLoader.hpp"
#include "util/String.hpp"

using namespace Dawn;

void const * ShaderManagerSlangString::getBufferPointer() {
  return this->str.c_str();
}

size_t ShaderManagerSlangString::getBufferSize() {
  return this->str.size();
}

SlangResult ShaderManagerSlangString::queryInterface(
  SlangUUID const& uuid,
  void** outObject
) {
  return SLANG_E_NOT_IMPLEMENTED;
}

uint32_t ShaderManagerSlangString::addRef() {
  return refs++;
}

uint32_t ShaderManagerSlangString::release() {
  return refs--;
}

//

SlangResult ShaderManagerSlangFileSystem::loadFile(
  const char* path,
  ISlangBlob** outBlob
) {
  auto shaderManager = this->sm.lock();
  assertNotNull(shaderManager, "ShaderManager must not be null.");
  
  if(!String::endsWith(path, ".slang")) return SLANG_E_NOT_FOUND;

  auto loader = shaderManager->getGame()->assetManager->get<StringLoader>(path);
  loader->loadImmediately();

  auto blob = new ShaderManagerSlangString();
  blob->str = loader->data;
  *outBlob = blob;

  return SLANG_OK;
}

SlangResult ShaderManagerSlangFileSystem::queryInterface(
  SlangUUID const& uuid,
  void** outObject
) {
  return SLANG_E_NOT_IMPLEMENTED;
}

uint32_t ShaderManagerSlangFileSystem::addRef() {
  return refs++;
}

uint32_t ShaderManagerSlangFileSystem::release() {
  return refs--;
}

void * ShaderManagerSlangFileSystem::castAs(SlangUUID const& uuid) {
  return nullptr;
}