// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawn.hpp"

namespace Dawn {
  typedef uint_fast32_t flag_t;

  class Flag final {
    public:
      template<typename T, typename J>
      static void turnOn(T &flag, const J check) {
        flag |= (T)check;
      }

      template<typename T, typename J>
      static void turnOff(T &flag, const J check) {
        flag &= ~((T)check);
      }

      template<typename T, typename J>
      static bool_t isOn(const T flag, const J check) {
        return (flag & (T)check) == (T)check;
      }

      template<typename T, typename J>
      static bool_t isOff(const T flag, const J check) {
        return (flag & (T)check) == 0;
      }
  };

  #define FLAG(i) ((flag_t)1 << (flag_t)i)
}