// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "String.hpp"

using namespace Dawn;

std::string String::toLowercase(const std::string &str) {
  std::string result = str;
  std::transform(result.begin(), result.end(), result.begin(), ::tolower);
  return result;
}

bool_t String::includes(const std::string &str, const std::string &needle) {
  return str.find(needle) != std::string::npos;
}

std::vector<std::string> String::split(
  const std::string &s,
  const std::string &delim
) {
  size_t posStart = 0, posEnd, delimLength = delim.length();
  std::string token;
  std::vector<std::string> res;

  while((posEnd = s.find(delim, posStart)) != std::string::npos) {
    token = s.substr(posStart, posEnd - posStart);
    posStart = posEnd + delimLength;
    res.push_back (token);
  }

  res.push_back(s.substr(posStart));
  return res;
}

bool_t String::endsWith(const std::string &str, const std::string &needle) {
  if(str.length() >= needle.length()) {
    return (0 == str.compare(
      str.length() - needle.length(), needle.length(), needle
    ));
  }
  return false;
}