// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "dawnopengl.hpp"
#include "assert/assert.hpp"
#include "assert/assertgl.hpp"
#include "util/Flag.hpp"
#include "BackBufferRenderTarget.hpp"

using namespace Dawn;

BackBufferRenderTarget::BackBufferRenderTarget() {

}

float_t BackBufferRenderTarget::getScale() {
  return this->scale;
}

float_t BackBufferRenderTarget::getWidth() {
  return this->width;
}

float_t BackBufferRenderTarget::getHeight() {
  return this->height;
}

void BackBufferRenderTarget::setSize(
  const float_t width,
  const float_t height
) {
  if(this->width == width && this->height == height) return;
  
  // Fixes a new bug that it seems GLFW has introduced.
  this->width = width == 0 ? 1 : width;
  this->height = height == 0 ? 1 : height;
  
  onResize.emit(this->width, this->height);
}

void BackBufferRenderTarget::setClearColor(const struct Color color) {
  this->clearColor = color;
}

void BackBufferRenderTarget::clear(const int32_t clearFlags) {
  glClearColor(clearColor.r, clearColor.g, clearColor.b, 1.0f);
  assertNoGLError();

  GLbitfield mask = 0;
  if(Flag::isOn(clearFlags, RENDER_TARGET_CLEAR_COLOR)) {
    mask |= GL_COLOR_BUFFER_BIT;
  }

  if(Flag::isOn(clearFlags, RENDER_TARGET_CLEAR_DEPTH)) {
    mask |= GL_DEPTH_BUFFER_BIT;
  }

  glClear(mask);
  assertNoGLError();
}

void BackBufferRenderTarget::bind() {
  glBindFramebuffer(GL_FRAMEBUFFER, 0);
  assertNoGLError();
  glViewport(0, 0, (GLsizei)this->width, (GLsizei)this->height);
  assertNoGLError();
}