// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnopengl.hpp"
#include "display/mesh/IMesh.hpp"

namespace Dawn {
  class Mesh : public IMesh {
    protected:
      GLuint vertexBuffer = -1;
      GLuint indexBuffer = -1;
      GLuint vertexArray = -1;

    public:
      void createBuffers(
        const int32_t verticeCount,
        const int32_t indiceCount
      ) override;

      void disposeBuffers() override;

      void bufferPositions(
        const int32_t pos,
        const glm::vec3 positions[],
        const int32_t len
      ) override;

      void bufferCoordinates(
        const int32_t pos,
        const glm::vec2 coordinates[],
        const int32_t len
      ) override;

      void bufferIndices(
        const int32_t pos,
        const int32_t indices[],
        const int32_t len
      ) override;

      void draw(
        const enum MeshDrawMode drawMode,
        const int32_t start,
        const int32_t count
      ) override;
      
      ~Mesh();
  };
}