// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "Game.hpp"
#include "component/SceneComponentRegistry.hpp"
#include "component/RPGEntity.hpp"
#include "component/RPGPlayer.hpp"
#include "component/RPGMap.hpp"

using namespace Dawn;

Game::Game() : IGame() {
  SceneComponentRegistry::reg<RPGEntity>("RPGEntity");
  SceneComponentRegistry::reg<RPGPlayer>("RPGPlayer");
  SceneComponentRegistry::reg<RPGMap>("RPGMap");
}

std::string Game::getInitialScene() {
  return "scenes/test_rpg_scene.json";
}

void Game::initManagers() {
  this->rpgManager = std::make_shared<RPGManager>();
  this->rpgManager->init(shared_from_this());
}

Game::~Game() {
  
}